(ns document-storage.git.versioning
  (:require [clj-jgit.porcelain :as jp]
            [clj-jgit.internal :refer [resolve-object]]
            [clojure.edn :refer [read-string]])
  (:import [java.io ByteArrayOutputStream PrintStream]))


(defn list-commits [repo path]
  (-> repo
      (jp/git-log :paths path)
      (->>
       (map #(let [blob (jp/get-blob repo (:id %) path)]
               (hash-map
                :commit (:id %)
                :date (get-in % [:committer :date])
                :version (jp/get-blob repo (:id %) path)
                :type (if blob
                        :update
                        :delete))))
       (sort-by :date)
       reverse)))

(defn load-revision [commit repo path]
  (let [git-repo   (.getRepository repo)
        blob-id (jp/get-blob-id repo commit path)
        out-buffer (ByteArrayOutputStream.)
        tmp-out (PrintStream. out-buffer true "UTF-8")]
    (.copyTo (.open git-repo blob-id) tmp-out)
    (read-string (.toString out-buffer))))
