(ns document-storage.git.views
  (:require [document-storage.git.repository :refer [repositories default-repository]]))

(defonce view-cache (atom {}))

(defn get-views [repository]
  (get-in @repositories [repository :views] {}))

(defn add-view [repository id function]
  (swap! repositories assoc-in [repository :views id] function))

(defn remove-view [repository id]
  (swap! repositories update-in [repository :views] dissoc id))

(defn load-views [id & {:keys [repository] :or {repository default-repository}}]
  (get-in @view-cache [repository id]))

(defn process-view [document view-fn]
  (try
    (view-fn document)
    (catch Exception e
      (str "Exception: " e))))

(defn generate-views [repository id document]
  (swap! view-cache update repository dissoc id)
  (doseq [[view-id view-fn] (get-views repository)]
    (swap! view-cache assoc-in [repository id view-id]
           (process-view document view-fn))))
